"""File for helpers"""

import wiz.constants
from wiz.errors import LicenseError

DEFAULT_LICENCES = {
    'MIT': wiz.constants.MIT_LICENSE,
    'BSD': wiz.constants.BSD_LICENSE
}


def write_license_file(license_type: str, author_name=None) -> str:
    """"
    :param license_name: An license name
    :param author_name: An author name, will added in an license file
    :return: An content of license
    """
    license_content = DEFAULT_LICENCES.get(license_type)
    if license_content is None:
        raise LicenseError

    author_name = author_name if author_name else 'Change me!'
    out_put_license = license_content.replace('Wizi Generator', author_name)
    return out_put_license



MIT_LICENSE = """
MIT License

Copyright (c) Wizi Generator

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


BSD_LICENSE = """
Copyright (c) Wizi Generator
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIEDi
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""



import click

from wiz.generator import WiziGenerator as Wizi


@click.command()
@click.option('-n', '--name', prompt='Enter project name',
              help='Create an project based on name.')
@click.option('-a', '--author', prompt='Enter author name',
              help='An author name')
@click.option('-lt', '--license_type', prompt='Type of license [MIT, BSD]',
              help='Type of license')
def create(name, author, license_type):
    """Generate an project"""
    wizi = Wizi(name)
    wizi.create_project(license_type=license_type, author_name=author)
    click.echo('%s was created.' % name)



"""Setup file"""

from distutils.core import setup

import wiz

VERSION = wiz.__version__
AUTHOR = wiz.__author__

setup_kwargs = {
    'name': 'wizi',
    'version': VERSION,
    'url': 'https://github.com/MichaelYusko/wizi',
    'license': 'MIT',
    'author': AUTHOR,
    'author_email': 'freshjelly12@yahoo.com',
    'description': 'Fast template generator for your project',
    'packages': ['wiz'],
    'classifiers': [
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License'
    ],
 }

requirements = ['click==6.7']
setup_kwargs['install_requires'] = requirements
setup(**setup_kwargs)

print(u"\n\n\t\t    "
      "Wizi version {} installation succeeded.\n".format(VERSION))



"""Base generator file"""
import abc
import os
import sys

from .helpers import write_license_file


class BaseGenerator:  # pylint: disable=too-few-public-methods
    """Base generator class"""
    def __init__(self, project_name):
        self.project_name = project_name

    @abc.abstractmethod
    def create(self):
        """Abstract method for all child classes"""
        pass

    def _make_path(self, file_name):
        """
        :param file_name: An file name
        :return: Path where, files would be saved
        """
        path = os.getcwd()
        return '{}/{}/{}'.format(path, self.project_name, file_name)


class FileGenerator(BaseGenerator):  # pylint: disable=too-few-public-methods
    """Generator class for an file"""
    def __init__(self, project_name):  # pylint: disable=useless-super-delegation
        super().__init__(project_name)

    _default_files = [
        'requirements.txt', '.travis.yml',
        'setup.py', 'tox.ini', 'LICENSE',
        'README.md', '.gitignore'
    ]

    def create_files(self, license_type, author_name):
        """
        Create an files,in the root directory of project
        and write License context, with an author name
        """
        for file_name in self._default_files:
            with open(self._make_path(file_name), 'w') as file:
                file.write('# Generated by Wizi')

        with open(self._make_path('LICENSE'), 'w') as file:
            write_license = write_license_file(license_type, author_name)
            file.write(write_license)


class DirectoryGenerator(BaseGenerator):  # pylint: disable=too-few-public-methods
    """Generator class for an directory"""
    def __init__(self, project_name):  # pylint: disable=useless-super-delegation
        super().__init__(project_name)

    def create(self):
        """Create an directory"""
        if os.path.isdir(self.project_name):
            print('The {} project already exists'.format(self.project_name))
            sys.exit(1)
        else:
            os.mkdir(self.project_name)


class WiziGenerator:  # pylint: disable=too-few-public-methods
    """Main class for generate flow"""
    def __init__(self, project_name):
        self.directory = DirectoryGenerator(project_name)
        self.files = FileGenerator(project_name)

    def create_project(self, license_type, author_name):
        """Start an project"""
        self.directory.create()
        self.files.create_files(license_type, author_name)
